import 'dart:math';

import 'package:flutter/material.dart';

import 'package:tetrisdual/entity/counter.dart';
import 'package:tetrisdual/layout/board_painter.dart';
import 'package:tetrisdual/provider/data.dart';

class Player {
  Player(this.playerId);
  int playerId;

  int _score = 0;
  int _currentTetrimino = 0;
  final Counter _counter = Counter();

  Widget buildTetriminoWidget(Data myProvider, double width) {
    return GestureDetector(
      onTapUp: (details) {
        if (playerId == myProvider.getCurrentPlayer().playerId) {
          pickRandomTetrimino();
          myProvider.redraw();
        }
      },
      child: CustomPaint(
        size: Size(width, width),
        willChange: false,
        painter: BoardPainter(_currentTetrimino),
        isComplex: true,
        key: Key(_currentTetrimino.toString()),
      ),
    );
  }

  Widget buildManagerWidget(Data myProvider) {
    return Expanded(
      child: Container(
        margin: const EdgeInsets.all(5),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: myProvider.currentPlayer == playerId
              ? [
                  _counter.buildCounterWidget(myProvider),
                  buildSubmitWidget(myProvider),
                ]
              : [],
        ),
      ),
    );
  }

  Widget buildSubmitWidget(Data myProvider) {
    const double gainFontSize = 70;

    const gainTestStyle = TextStyle(
      fontFamily: 'Blocks',
      fontSize: gainFontSize,
      fontWeight: FontWeight.bold,
    );
    const submitIcon = Icon(
      Icons.done_all,
      color: Colors.orange,
      size: gainFontSize / 2,
    );

    const topBorderBlack = BoxDecoration(
      border: Border(
        top: BorderSide(
          color: Colors.black,
          width: 2,
        ),
      ),
    );

    return Container(
      decoration: topBorderBlack,
      padding: const EdgeInsets.only(
        top: 10,
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Text(
            '+${_counter.computePoints()}',
            style: gainTestStyle,
            textHeightBehavior: const TextHeightBehavior(
              applyHeightToFirstAscent: false,
              applyHeightToLastDescent: false,
            ),
          ),
          const SizedBox(width: 10),
          IconButton(
            padding: EdgeInsets.zero,
            constraints: const BoxConstraints(),
            style: const ButtonStyle(
              tapTargetSize: MaterialTapTargetSize.shrinkWrap,
            ),
            icon: submitIcon,
            onPressed: () {
              _score = _score + _counter.computePoints();
              _counter.reset();
              myProvider.toggleCurrentPlayer();
            },
          ),
          const SizedBox(width: 10),
        ],
      ),
    );
  }

  Widget buildPlayerBoard(Data myProvider, double screenWidth, bool isActive) {
    final double tetriminoBlockWidth = screenWidth / 2.3;
    final Color borderColor = isActive ? Colors.greenAccent : Colors.blueGrey;
    const double borderWidth = 10;

    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.end,
      children: [
        Text(
          _score.toString(),
          style: const TextStyle(
            fontFamily: 'Blocks',
            fontSize: 130,
            fontWeight: FontWeight.bold,
          ),
          textHeightBehavior: const TextHeightBehavior(
            applyHeightToFirstAscent: false,
            applyHeightToLastDescent: false,
          ),
        ),
        Container(
          decoration: BoxDecoration(
            border: Border.all(
              color: borderColor,
              width: borderWidth,
            ),
          ),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              isActive
                  ? buildTetriminoWidget(myProvider, tetriminoBlockWidth)
                  : SizedBox(
                      height: tetriminoBlockWidth,
                    ),
              buildManagerWidget(myProvider),
            ],
          ),
        )
      ],
    );
  }

  void pickRandomTetrimino() {
    // ensure new tetrimino is not same as current one
    int newTetrimino = _currentTetrimino;
    while (newTetrimino == _currentTetrimino) {
      newTetrimino = Random().nextInt(5) + 1;
    }

    _currentTetrimino = newTetrimino;
  }

  void resetTetrimino() {
    _currentTetrimino = 0;
  }

  void submitPoints() {
    _score = _score + _counter.computePoints();
    _counter.reset();
  }
}
